import org.apache.spark.mllib.linalg.{Vector, Vectors}

import scala.collection.mutable.{ArrayBuffer, HashMap, HashSet, ListBuffer}
import scala.util.control.Breaks.{break, breakable}


/**
  * This class implements the hypervolume indicator. The code is the a Java version
  * of the original metric implementation by Eckart Zitzler.
  * It can be used also as a command line program just by typing
  * $java jmetal.qualityIndicator.Hypervolume <solutionFrontFile> <trueFrontFile> <numberOfOjbectives>
  * Reference: E. Zitzler and L. Thiele
  *           Multiobjective Evolutionary Algorithms: A Comparative Case Study
  *           and the Strength Pareto Approach,
  *           IEEE Transactions on Evolutionary Computation, vol. 3, no. 4,
  *           pp. 257-271, 1999.
  */

/**
  * This class can be invoqued from the command line. Three params are required:
  * 1) the name of the file containing the front,
  * 2) the name of the file containig the true Pareto front
  * 3) the number of objectives
  */
object Hypervolume {

  //public jmetal.qualityIndicator.util.MetricsUtil utils_;

  val referenceParetoFront  = 1
  var PF : ListBuffer[Double] = new ListBuffer[Double]()

  /**
    * Constructor
    * Creates a new instance of MultiDelta
    */
  def Hypervolume()
  {
    //utils_ = new jmetal.qualityIndicator.util.MetricsUtil();
  } // Hypervolume

  /*
   returns true if 'point1' dominates 'points2' with respect to the
   to the first 'noObjectives' objectives
   */
  def  dominates(x : Vector, y : Vector, noObjectives: Int) : Boolean =
  {

    var betterInAnyObjective : Boolean = false;
    var worseInAnyObjective : Boolean = false;
    breakable {
      for (i <- 0 until y.toArray.length)
      {

        if (worseInAnyObjective)
        {
          break;
        }

        if ( i == 0 | i == 2 )
        {
          if ( x(i) > y(i) )
          {
            betterInAnyObjective = true;
          }

          else if (x(i) < y(i))
          {
            worseInAnyObjective = true;
          }
        }
        else
        {
          if ( x(i) < y(i) )
          {
            betterInAnyObjective = true;
          }

          else if (x(i) > y(i))
          {
            worseInAnyObjective = true;
          }
        }

      }
    }

    return !worseInAnyObjective && betterInAnyObjective;

    //return !worseInAnyObjective && betterInAnyObjective;

    ////////////End
    /*var  i: Int = 0
    var betterInAnyObjective: Int = 0

    betterInAnyObjective = 0;
    for (i = 0; i < noObjectives && point1[i] >= point2[i]; i++)
    if (point1[i] > point2[i])
      betterInAnyObjective = 1;

    return ((i >= noObjectives) && (betterInAnyObjective>0));*/
  } //Dominates

  def  swap( front: ListBuffer[Vector], i: Int, j: Int ) =
  {
    var temp = front(i).copy
    //temp = ;
    front(i) = front(j).copy;
    front(j) = temp.copy;
  } // Swap


  /* all nondominated points regarding the first 'noObjectives' dimensions
  are collected; the points referenced by 'front[0..noPoints-1]' are
  considered; 'front' is resorted, such that 'front[0..n-1]' contains
  the nondominated points; n is returned */
  def filterNondominatedSet(front : ListBuffer[Vector], noPoints: Int, noObjectives: Int) : Int =
  {
    var  i , j: Int = 0
    var  n : Int = noPoints
    i = 0
    while (i < n)
    {
      j = i + 1;
      while (j < n)
      {
        if (dominates(front(i), front(j), noObjectives))
        {
          /* remove point 'j' */
          n -= 1;
          swap(front, j, n);
        } else if (dominates(front(j), front(i), noObjectives))
        {
          /* remove point 'i'; ensure that the point copied to index 'i'
             is considered in the next outer loop (thus, decrement i) */
          n-= 1;
          swap(front, i, n);
          i-= 1;
          break;
        }
        else
          j+= 1;
      }
      i+= 1;
    }
    return n;
  } // FilterNondominatedSet


  /* calculate next value regarding dimension 'objective'; consider
     points referenced in 'front[0..noPoints-1]' */
  def  surfaceUnchangedTo(front: ListBuffer[Vector], noPoints: Int, objective: Int) : Double =
  {
    var i: Int  = 0
    var  minValue, value: Double = 0.0

    if (noPoints < 1)
      System.err.println("run-time error");

    minValue = front(0).toArray(objective)
    for (i <- 1 until  noPoints  )
    {
      value = front(i).toArray(objective)
      if (value < minValue)
        minValue = value

      /*if ( objective == 0 | objective == 2 )
      {
        //Maximization
        if ( value > minValue )
          minValue = value;
      }
      else
      {
        //Minimization
        if (value < minValue)
          minValue = value;
      }*/

    }
    return minValue;
  } // SurfaceUnchangedTo

  /* remove all points which have a value <= 'threshold' regarding the
     dimension 'objective'; the points referenced by
     'front[0..noPoints-1]' are considered; 'front' is resorted, such that
     'front[0..n-1]' contains the remaining points; 'n' is returned */
  def reduceNondominatedSet( front: ListBuffer[Vector], noPoints: Int, objective: Int, threshold: Double) : Int =
  {
    var i, n: Int = 0
    n = noPoints

    for (i <- 0 until n )
      {
        if (front(i).toArray(objective) <= threshold)
        {
          n-= 1;
          swap(front, i, n);
        }
      }

    return n;
  } // ReduceNondominatedSet

  def calculateHypervolume( front: ListBuffer[Vector], noPoints: Int, noObjectives: Int): Double =
  {
    var n: Int = noPoints
    var volume, distance : Double = 0.0

    while (n > 0)
    {
      var noNondominatedPoints: Int = 0
      var tempVolume, tempDistance: Double = 0.0

      noNondominatedPoints = filterNondominatedSet(front, n, noObjectives - 1);
      //noNondominatedPoints = front.length;
      if (noObjectives < 3)
      {
        if (noNondominatedPoints < 1)
          System.err.println("run-time error");

        tempVolume = front(0).toArray(0);
      } else
        tempVolume = calculateHypervolume(front, noNondominatedPoints, noObjectives - 1);

      tempDistance = surfaceUnchangedTo(front, n, noObjectives - 1);
      volume += tempVolume * (tempDistance - distance);
      distance = tempDistance;
      n = reduceNondominatedSet(front, n, noObjectives - 1, distance);
    }
    return volume;
  } // CalculateHypervolume


  /* merge two fronts */
  def mergeFronts(front1: ListBuffer[Vector], sizeFront1: Int,
    front2: ListBuffer[Vector], sizeFront2: Int, noObjectives: Int) : ListBuffer[Vector] =
  {
    var i, j: Int = 0
    var noPoints: Int = 0
    var frontPtr: ListBuffer[Vector] = new ListBuffer[Vector]()

    /* allocate memory */
    noPoints = sizeFront1 + sizeFront2;
    //frontPtr = new double[noPoints][noObjectives];
    /* copy points */
    noPoints = 0;
    for (i <- 0 until sizeFront1 )
    {
      frontPtr += front1(i)
      /*for (j <- 0 until noObjectives )
        {   frontPtr(noPoints)(j) = front1(i)(j);   }
      noPoints+=1;*/
    }
    for (i <- 0 until sizeFront2)
    {
      frontPtr += front2(i)
      /*for (j <- 0 until noObjectives )
        frontPtr(noPoints)(j) = front2(i)(j);
      noPoints+=1;*/
    }

    return frontPtr;
  } // MergeFronts

  /**
    * Returns the hypevolume value of the paretoFront. This method call to the
    * calculate hipervolume one
    * @param paretoFront The pareto front
    * @param paretoTrueFront The true pareto front
    * @param numberOfObjectives Number of objectives of the pareto front
    */
  def hypervolume( paretoFront: ListBuffer[Vector], paretoTrueFront : ListBuffer[Vector], numberOfObjectives: Int) : Double =
  {

    /**
      * Stores the maximum values of true pareto front.
      */
    var maximumValues: ListBuffer[Double] = new ListBuffer[Double]()

    /**
      * Stores the minimum values of the true pareto front.
      */
    var minimumValues: ListBuffer[Double] = new ListBuffer[Double]()


    /**
      * Stores the normalized front.
      */
    //var normalizedFront: ListBuffer[Vector] = new ListBuffer[Vector]()

    /**
      * Stores the inverted front. Needed for minimization problems
      */
    //double [][] invertedFront;
    var invertedFront: ListBuffer[Vector] = new ListBuffer[Vector]()

    // STEP 1. Obtain the maximum and minimum values of the Pareto front
    maximumValues = getMaximumValues(paretoTrueFront,numberOfObjectives);
    minimumValues = getMinimumValues(paretoTrueFront,numberOfObjectives);

    // STEP 2. Get the normalized front
    var normalizedFront = getNormalizedFront(paretoFront, maximumValues, minimumValues);

    // STEP 3. Inverse the pareto front. This is needed because of the original
    //metric by Zitzler is for maximization problems
    invertedFront = invertedFrontFun(normalizedFront);
    //invertedFront = normalizedFront

    // STEP4. The hypervolumen (control is passed to java version of Zitzler code)
    return calculateHypervolume(invertedFront,invertedFront.length,numberOfObjectives);
  }// hypervolume

  /**
    * This method receives a normalized pareto front and return the inverted one.
    * This operation needed for minimization problems
    * @param front The pareto front to inverse
    * @return The inverted pareto front
    **/
  def invertedFrontFun( front: ListBuffer[Vector]) : ListBuffer[Vector] =
  {
    var invertedFront: ListBuffer[Vector] = new ListBuffer[Vector]

    for ( i <- 0 until  front.length )
    {
      invertedFront  += front(i).copy

      for ( j <- 0 until front(i).toArray.length )
      {
        if (j == 0 | j == 2) {
          if (front(i)(j) <= 1.0 & front(i)(j) >= 0.0) {
            invertedFront(i).toArray.update(j, 1.0 - front(i)(j)); // (i)(j)
          }
          else if (front(i)(j) > 1.0) {
            //invertedFront(i)(j) = 0.0;
            invertedFront(i).toArray.update(j, 0.0);
          }
          else if (front(i)(j) < 0.0) {
            //invertedFront(i)(j) = 1.0;
            invertedFront(i).toArray.update(j, 1.0 );
          }
        }
      }
    }

    return invertedFront;
  } // invertedFront


  /** Gets the maximum values for each objectives in a given pareto
    *  front
    *  @param front The pareto front
    *  @param noObjectives Number of objectives in the pareto front
    *  @return double [] An array of noOjectives values whit the maximun values
    *  for each objective
    **/
  def getMaximumValues( front: ListBuffer[Vector], noObjectives: Int) : ListBuffer[Double] =
  {
    var maximumValue : ListBuffer[Double] = new ListBuffer[Double]()
    for ( i <- 0 until noObjectives )
    maximumValue +=  Double.NegativeInfinity// .NEGATIVE_INFINITY;

    for (aFront <- front )
      {
        for ( j <- 0 until aFront.toArray.length )
          {
            if (aFront(j) > maximumValue(j))
              {       maximumValue(j) = aFront(j)      }
          }
      }

    return maximumValue;
  } // getMaximumValues


  /** Gets the minimum values for each objectives in a given pareto
    *  front
    *  @param front The pareto front
    *  @param noObjectives Number of objectives in the pareto front
    *  @return double [] An array of noOjectives values whit the minimum values
    *  for each objective
    **/
  def getMinimumValues( front: ListBuffer[Vector], noObjectives: Int) : ListBuffer[Double] =
  {
    var minimumValue : ListBuffer[Double] = new ListBuffer[Double]()
    for ( i <- 0 until noObjectives )
      minimumValue +=  Double.MaxValue// .MAX_VALUE;

    for (aFront <- front )
    {
      for ( j <- 0 until aFront.toArray.length )
      {
        if (aFront(j) < minimumValue(j))
        {       minimumValue(j) = aFront(j)      }
      }
    }

    return minimumValue;
  } // getMinimumValuess


  /**
    * This method receives a pareto front and two points, one whit maximum values
    * and the other with minimum values allowed, and returns a the normalized
    * Pareto front.
    * @param front A pareto front.
    * @param maximumValue The maximum values allowed
    * @param minimumValue The minimum values allowed
    * @return the normalized pareto front
    **/
  def getNormalizedFront( front : ListBuffer[Vector], maximumValue: ListBuffer[Double],
    minimumValue: ListBuffer[Double]) : ListBuffer[Vector] =
  {

    var normalizedFront : ListBuffer[Vector] = new ListBuffer[Vector]  // double[front.length][];
    var l : Int = front(0).toArray.length

    for ( i <- 0  until front.length )
    {
      normalizedFront += Vectors.zeros(l)  //Vectors.dense  //double[front[i].length];
      for ( j <- 0 until front(i).toArray.length )
      {
        normalizedFront(i).toArray(j) = (front(i).toArray(j) - minimumValue(j)) /
          (maximumValue(j) - minimumValue(j)).asInstanceOf[Double];
      }
    }

    return normalizedFront;
  } // getNormalizedFront

  /**
    * This method receives a normalized pareto front and return the inverted one.
    * This operation needed for minimization problems
    * @param front The pareto front to inverse
    * @return The inverted pareto front
    **/
  def invertedFront( front: ListBuffer[Vector]) : ListBuffer[Vector] =
  {

    var invertedFront : ListBuffer[Vector] = new ListBuffer[Vector]
    //double [][] invertedFront = new double[front.length][];

    var l : Int = front(0).toArray.length
    for ( i <- 0  until front.length )
    {
      invertedFront += Vectors.zeros(l) //new double[front[i].length];
      for ( j <- 0 until front(i).toArray.length )
      {
        if (front(i).toArray(j) <= 1.0 & front(i).toArray(j)>= 0.0)
        {
          invertedFront(i).toArray(j) = 1.0 - front(i).toArray(j);
        }
        else if (front(i).toArray(j) > 1.0)
        {
        invertedFront(i).toArray(j) = 0.0;
        }
        else if (front(i).toArray(j) < 0.0)
        {
          invertedFront(i).toArray(j) = 1.0;
        }
      }
    }

    return invertedFront;
  } // invertedFront



  //////////////////////
  ////////////////////
  //////////////////
  /**
    * Calculates how much hypervolume each point dominates exclusively. The points
    * have to be transformed beforehand, to accommodate the assumptions of Zitzler's
    * hypervolume code.
    * @param front transformed objective values
    * @return HV contributions
    */
  def hvContributions(numberOfObjectives: Int, front: ListBuffer[Vector]) : ListBuffer[Double] =
  {

    //Hypervolume hypervolume = new Hypervolume() ;
    //int numberOfObjectives = numberOfobjectives;

    //double[] contributions = new double[front.length];
    var contributions : ListBuffer[Double] = new ListBuffer[Double]
    for ( i <- 0 until front.length )
      {
        contributions += 0.0
      }

    //double[][] frontSubset = new double[front.length - 1][front[0].length];
    var frontSubset : ListBuffer[Vector] = new ListBuffer[Vector]
    for ( i <- 0 until front.length-1 )
    {
      frontSubset += Vectors.zeros(front(0).toArray.length)
    }


    //LinkedList<double[]> frontCopy = new LinkedList<double[]>();
    var frontCopy : ListBuffer[Vector] = new ListBuffer[Vector]

    //Collections.addAll(frontCopy, front);
    for ( a <- front )
      {
        frontCopy += a
      }

    //double[][] totalFront = frontCopy.toArray(frontSubset);
    var totalFront : ListBuffer[Vector] = new ListBuffer[Vector]
    for ( a <- front )
    {
      totalFront += a
    }
    //double totalVolume = hypervolume.calculateHypervolume(totalFront, totalFront.length, numberOfObjectives);
    var totalVolume : Double = calculateHypervolume(totalFront, totalFront.length, numberOfObjectives);

    for ( i <- 0 until front.length )
    {
      var evaluatedPoint : Vector = frontCopy(i).copy
      frontCopy.remove(i)
      // STEP4. The hypervolume (control is passed to java version of Zitzler code)
      //frontSubset = frontCopy.toArray(frontSubset);
      for ( a <- front )
      {
        frontSubset += a
      }
    // STEP4. The hypervolume (control is passed to java version of Zitzler code)
    var hv : Double = calculateHypervolume(frontSubset, frontSubset.length, numberOfObjectives);
    var contribution : Double = totalVolume - hv;
    contributions(i) = contribution;
    // put point back
    //frontCopy.add(i, evaluatedPoint);
      frontCopy += evaluatedPoint
    }

    /*for (int i = 0; i < front.length; i++) {
      double[] evaluatedPoint = frontCopy.remove(i);
      frontSubset = frontCopy.toArray(frontSubset);
      // STEP4. The hypervolume (control is passed to java version of Zitzler code)
      double hv = hypervolume.calculateHypervolume(frontSubset, frontSubset.length, numberOfObjectives);
      double contribution = totalVolume - hv;
      contributions[i] = contribution;
      // put point back
      frontCopy.add(i, evaluatedPoint);
    }*/
    return contributions;
  }

/*
  /**
    * Calculates the hv contribution of different populations.
    * Receives an array of populations and computes the contribution to HV of the
    * population consisting in the union of all of them
    * @param populations, consisting in all the populatoins
    * @return HV contributions of each population
    **/
  def  hvContributions(SolutionSet [] populations) : ListBuffer[Double] =
  {
    var empty : Boolean = true;
    for (SolutionSet population2 : populations)
    if (population2.size() > 0)
      empty = false;

    if (empty) {
      double [] contributions = new double[populations.length];
      for (int i = 0; i < populations.length;i++)
      contributions[i]=0;
      for (int i = 0; i < populations.length; i++)
      System.out.println(contributions[i]);
      return contributions;
    }




    SolutionSet union;
    int size = 0;
    double  offset_= 0.0;

    //determining the global size of the population
    for (SolutionSet population1 : populations) size += population1.size();

    //allocating space for the union
    union = new SolutionSet(size);

    // filling union
    for (SolutionSet population : populations)
    for (int j = 0; j < population.size(); j++)
    union.add(population.get(j));


    //determining the number of objectives
    int numberOfObjectives = union.get(0).getNumberOfObjectives();

    //writing everything in matrices
    double[][][] frontValues = new double[populations.length+1][][];

    frontValues[0] = union.writeObjectivesToMatrix();
    for (int i = 0; i < populations.length; i++)
    if (populations[i].size() > 0)
      frontValues[i+1] = populations[i].writeObjectivesToMatrix();
    else
    frontValues[i+1] = new double[0][];


    // obtain the maximum and minimum values of the Pareto front
    double[] maximumValues = getMaximumValues(union.writeObjectivesToMatrix(), numberOfObjectives);
    double[] minimumValues = getMinimumValues(union.writeObjectivesToMatrix(), numberOfObjectives);


    // normalized all the fronts
    double[][][] normalizedFront = new double[populations.length+1][][];
    for (int i = 0; i < normalizedFront.length; i++) {
      if (frontValues[i].length > 0)
        normalizedFront[i] = getNormalizedFront(frontValues[i],maximumValues,minimumValues);
      else
        normalizedFront[i] = new double[0][];
    }

    // compute offsets for reference point in normalized space
    double[] offsets = new double[maximumValues.length];
    for (int i = 0; i < maximumValues.length; i++) {
      offsets[i] = offset_ / (maximumValues[i] - minimumValues[i]);
    }

    //Inverse all the fronts front. This is needed because the original
    //metric by Zitzler is for maximization problems

    double[][][] invertedFront = new double[populations.length+1][][];
    for (int i = 0; i < invertedFront.length; i++)
    if (normalizedFront[i].length > 0)
      invertedFront[i] = invertedFront(normalizedFront[i]);
    else
      invertedFront[i] = new double[0][];

    // shift away from origin, so that boundary points also get a contribution > 0
    for (double[][] anInvertedFront : invertedFront) {
      for (double[] point : anInvertedFront) {
        for (int i = 0; i < point.length; i++) {
          point[i] += offsets[i];
        }
      }
    }

    // calculate contributions
    double [] contribution = new double[populations.length];
    Hypervolume hypervolume = new Hypervolume();

    for (int i = 0; i < populations.length;i++) {
      if (invertedFront[i+1].length == 0)
      contribution[i] = 0;
      else
      {
        if (invertedFront[i+1].length!=invertedFront[0].length) {
        double [][] aux = new double[invertedFront[0].length - invertedFront[i+1].length][];
        int startPoint = 0, endPoint;
        for (int j = 0; j < i; j++) {
          startPoint += invertedFront[j+1].length;
        }
        endPoint = startPoint + invertedFront[i+1].length;
        int index = 0;
        for (int j = 0; j < invertedFront[0].length; j++) {
          if (j < startPoint || j >= (endPoint)) {
            aux[index++]= invertedFront[0][j];
          }
        }
        //System.out.println(hypervolume.calculateHypervolume(invertedFront[0], invertedFront[0].length, getNumberOfObjectives));
        //System.out.println(hypervolume.calculateHypervolume(aux, aux.length, getNumberOfObjectives));

        contribution[i] = hypervolume.calculateHypervolume(invertedFront[0], invertedFront[0].length, numberOfObjectives) -
          hypervolume.calculateHypervolume(aux, aux.length, numberOfObjectives);
      } else {
        contribution[i] = hypervolume.calculateHypervolume(invertedFront[0], invertedFront[0].length, numberOfObjectives);
      }
      }
    }

    //for (int i = 0; i < contribution.length; i++)
    //System.out.println(invertedFront[0].length +" "+ invertedFront[i+1].length +" "+ contribution[i]);

    return contribution;
  }



  /**
    * Calculates the hv contribution of different populations.
    * Receives an array of populations and computes the contribution to HV of the
    * population consisting in the union of all of them
    * @param populations, consisting in all the populatoins
    * @return HV contributions of each population
    **/
  public double[] hvContributions(SolutionSet archive, SolutionSet [] populations) {

    SolutionSet union;
    int size = 0;
    double  offset_= 0.0;

    //determining the global size of the population
    for (SolutionSet population : populations) size += population.size();

    //allocating space for the union
    union = archive;

    //determining the number of objectives
    int numberOfObjectives = union.get(0).getNumberOfObjectives();

    //writing everything in matrices
    double[][][] frontValues = new double[populations.length+1][][];

    frontValues[0] = union.writeObjectivesToMatrix();
    for (int i = 0; i < populations.length; i++)
    if (populations[i].size() > 0)
      frontValues[i+1] = populations[i].writeObjectivesToMatrix();
    else
    frontValues[i+1] = new double[0][];


    // obtain the maximum and minimum values of the Pareto front
    double[] maximumValues = getMaximumValues(union.writeObjectivesToMatrix(), numberOfObjectives);
    double[] minimumValues = getMinimumValues(union.writeObjectivesToMatrix(), numberOfObjectives);


    // normalized all the fronts
    double[][][] normalizedFront = new double[populations.length+1][][];
    for (int i = 0; i < normalizedFront.length; i++) {
      if (frontValues[i].length > 0)
        normalizedFront[i] = getNormalizedFront(frontValues[i],maximumValues,minimumValues);
      else
        normalizedFront[i] = new double[0][];
    }

    // compute offsets for reference point in normalized space
    double[] offsets = new double[maximumValues.length];
    for (int i = 0; i < maximumValues.length; i++) {
      offsets[i] = offset_ / (maximumValues[i] - minimumValues[i]);
    }

    //Inverse all the fronts front. This is needed because the original
    //metric by Zitzler is for maximization problems

    double[][][] invertedFront = new double[populations.length+1][][];
    for (int i = 0; i < invertedFront.length; i++)
    if (normalizedFront[i].length > 0)
      invertedFront[i] = invertedFront(normalizedFront[i]);
    else
      invertedFront[i] = new double[0][];

    // shift away from origin, so that boundary points also get a contribution > 0
    for (double[][] anInvertedFront : invertedFront) {
      for (double[] point : anInvertedFront) {
        for (int i = 0; i < point.length; i++) {
          point[i] += offsets[i];
        }
      }
    }

    // calculate contributions
    double [] contribution = new double[populations.length];
    Hypervolume hypervolume = new Hypervolume();

    for (int i = 0; i < populations.length;i++) {
      if (invertedFront[i+1].length == 0)
      contribution[i] = 0;
      else
      {


        int auxSize = 0;
        for (int j = 0; j < populations.length; j++) {
        if (j!=i)
          auxSize += invertedFront[j+1].length;
      }

        if (size == archive.size()) { // the contribution is the maximum hv
          contribution[i] = hypervolume.calculateHypervolume(invertedFront[0], invertedFront[0].length, numberOfObjectives);
        } else {
          //make a front with all the populations but the target one
          int index = 0;
          double [][] aux = new double[auxSize][];
          for (int  j = 0; j < populations.length; j++) {
            if (j!=i)
              for (int k = 0; k < populations[j].size(); k++)
            aux[index++] = invertedFront[j+1][k];
          }
          contribution[i] = hypervolume.calculateHypervolume(invertedFront[0], invertedFront[0].length, numberOfObjectives) -
            hypervolume.calculateHypervolume(aux, aux.length, numberOfObjectives);
        }



        /*
          int size2 = 0;
          for (int j = 0; j < populations.length; j++)
            size2+=invertedFront[j+1].length;


          double [][] aux = new double[size2 - invertedFront[i+1].length][];
          int index = 0;
          for (int j = 0; j < populations.length; j++) {
            if (j!=i) {
              for (int k = 0; k < invertedFront[j+1].length; k++)
                aux[index++] = invertedFront[j+1][k];
            }
          }

          System.out.println(hypervolume.calculateHypervolume(invertedFront[0], invertedFront[0].length, getNumberOfObjectives));
          System.out.println(index+" "+aux.length);
          System.out.println(hypervolume.calculateHypervolume(aux, aux.length, getNumberOfObjectives));



          contribution[i] = hypervolume.calculateHypervolume(invertedFront[0], invertedFront[0].length, getNumberOfObjectives) -
                  hypervolume.calculateHypervolume(aux, aux.length, getNumberOfObjectives);
        */

      }
    }

    //for (int i = 0; i < contribution.length; i++)
    //System.out.println(invertedFront[0].length +" "+ invertedFront[i+1].length +" "+ contribution[i]);

    return contribution;
  } */

} // Hypervolume
